package internet

import (
	"fmt"
	"os"
	"os/exec"
	"strings"

	"gitea.unprism.ru/KRBL/sim-modem/api/modem/utils"
)

var apns = map[string]string{
	"Tinkoff": "m.tinkoff",
	"Megafon": "internet",
	"BeeLine": "internet.beeline.ru",
}

const pppConfigName = "annalistnet"
const pppConfig = `
# Annalist project custom internet connection

# APN:
connect "/usr/sbin/chat -v -f /etc/chatscripts/gprs -T %s"

# Port:
%s

# Baudrate:
%d

noipdefault
usepeerdns
defaultroute
persist
noauth
nocrtscts
local
`

func (c *conn) setup() error {
	if err := c.checkReqs(); err != nil {
		return fmt.Errorf("check requirenments: %w", err)
	}
	// Configure ppp
	// what is better ASK If /etc/ppp/peers/annalistnet not exists
	if err := c.configurePPP(); err != nil {
		return fmt.Errorf("configure ppp: %w", err)
	}
	return nil
}

// Check requirenments
func (c *conn) checkReqs() error {
	// Check AT port for sure
	if c.port == nil || !c.port.IsConnected() {
		return fmt.Errorf("AT port is not connect or nil")
	}

	// Ensure all necessary packages installed
	if err := c.ensurePackage("ppp"); err != nil {
		return fmt.Errorf("ensure ppp package: %w", err)
	}
	// if err := c.ensurePackage("net-tools"); err != nil {
	// 	return fmt.Errorf("ensure net-tools package: %w", err)
	// }

	// Check SIM is valid
	if err := utils.CheckPIN(c.port, c.logger); err != nil {
		return fmt.Errorf("PIN check: %w", err)
	}
	return nil
}

func (c *conn) ensurePackage(pname string) error {
	if c.checkPackageExist(pname) {
		return nil
	}
	return fmt.Errorf("package %s not installed", pname)
	// c.logger.Println("Installing", pname, "package...")
	// resp, err := exec.Command("apt-get", "install", pname).Output()
	// if err != nil {
	// return fmt.Errorf("execute install cmd: %w", err)
	// }
	// c.logger.Println(resp)
	// c.logger.Println("\x1b[38;2;255;0;0mComplete\x1b[38;2;255;255;255m")
	// return nil
}

func (c *conn) checkPackageExist(pname string) bool {
	resp, err := exec.Command("apt-mark", "showmanual", pname).Output()
	if err != nil {
		c.logger.Println("check package error: ", err.Error())
		return false
	}
	return string(resp[:len(pname)]) == pname
}

func (c *conn) configurePPP() error {
	// Get provider name and its APN
	resp, err := c.port.Send("AT+CSPN?")
	if err != nil {
		return fmt.Errorf("AT+CSPN? request: %w", err)
	}
	if !resp.Check() {
		return fmt.Errorf("get provider: error response: %s", resp)
	}
	strs := strings.Split(string(resp), "\"")
	if len(strs) < 3 {
		return fmt.Errorf("parse AT+CSPN response: %s", string(resp))
	}
	provider := strs[1]
	apn := apns[provider]
	if apn == "" {
		return fmt.Errorf("no apn for provider: %s", provider)
	}

	// Make config
	c.logger.Printf("Config ppp values: %s, %s, %d", apn, c.pppPort, c.port.GetBaudrate())
	config := fmt.Sprintf(pppConfig, apn, c.pppPort, c.port.GetBaudrate())

	// Write to file
	f, err := os.OpenFile("/etc/ppp/peers/"+pppConfigName, os.O_CREATE|os.O_WRONLY, 0666)
	if err != nil {
		return fmt.Errorf("open ppp config file %w", err)
	}
	defer f.Close()
	if _, err := f.Write([]byte(config)); err != nil {
		return fmt.Errorf("write to ppp config file: %w", err)
	}
	return nil
}
